@@ -5,6 +5,8 @@ Notes:
       on third party providers that (if using POST) won't be sending csrf
       token back.
 """
+from urllib2 import quote
+
 from django.http import HttpResponseRedirect, HttpResponse
 from django.contrib.auth import login, REDIRECT_FIELD_NAME
 from django.contrib.auth.decorators import login_required
@@ -13,7 +15,7 @@ from django.views.decorators.csrf import csrf_exempt
 
 from social_auth.utils import sanitize_redirect, setting, \
                               backend_setting, clean_partial_pipeline
-from social_auth.decorators import dsa_view
+from social_auth.decorators import dsa_view, disconnect_view
 
 
 DEFAULT_REDIRECT = setting('SOCIAL_AUTH_LOGIN_REDIRECT_URL',
@@ -60,6 +62,7 @@ def associate_complete(request, backend, *args, **kwargs):
 
 @login_required
 @dsa_view()
+@disconnect_view
 def disconnect(request, backend, association_id=None):
     """Disconnects given backend from current logged in user."""
     backend.disconnect(request.user, association_id)
@@ -93,7 +96,8 @@ def auth_process(request, backend):
 def complete_process(request, backend, *args, **kwargs):
     """Authentication complete process"""
     # pop redirect value before the session is trashed on login()
-    redirect_value = request.session.get(REDIRECT_FIELD_NAME, '')
+    redirect_value = request.session.get(REDIRECT_FIELD_NAME, '') or \
+                     request.REQUEST.get(REDIRECT_FIELD_NAME, '')
     user = auth_complete(request, backend, *args, **kwargs)
 
     if isinstance(user, HttpResponse):
@@ -152,6 +156,13 @@ def complete_process(request, backend, *args, **kwargs):
         url = backend_setting(backend, 'LOGIN_ERROR_URL', LOGIN_ERROR_URL)
     if msg:
         messages.error(request, msg)
+
+    if redirect_value and redirect_value != url:
+        redirect_value = quote(redirect_value)
+        if '?' in url:
+            url += '&%s=%s' % (REDIRECT_FIELD_NAME, redirect_value)
+        else:
+            url += '?%s=%s' % (REDIRECT_FIELD_NAME, redirect_value)
     return HttpResponseRedirect(url)
 
 
